@@ -29,7 +29,7 @@ import os
 import pandas as pd
 from mitosheet.user import initialize_user
 from mitosheet.mito_widget import MitoWidget, sheet
-from mitosheet.save_utils import register_analysis
+from mitosheet.saved_analyses import register_analysis
 from mitosheet.errors import CreationError, EditError
 from mitosheet._version import __version__
 from mitosheet.utils import make_valid_header
