@@ -2,7 +2,7 @@ from mitosheet.sheet_functions.types.utils import NUMBER_SERIES, get_mito_type
 import pandas as pd
 import json
 
-def get_column_describe(event, wsc):
+def get_column_describe(event, steps_manager):
     """
     Sends back a string that can be parsed to a JSON object that
     contains _all_ the results from the series .describe function
@@ -11,7 +11,7 @@ def get_column_describe(event, wsc):
     sheet_index = event['sheet_index']
     column_header = event['column_header']
     
-    series: pd.Series = wsc.dfs[sheet_index][column_header]
+    series: pd.Series = steps_manager.dfs[sheet_index][column_header]
     describe = series.describe()
 
     try:
