@@ -653,7 +653,7 @@ def get_column_summary_graph(axis, df, axis_data_array):
     # we always return False as the second parameter
     return fig, False
 
-def get_graph(event, wsc):
+def get_graph(event, steps_manager):
     """
     Creates a graph of the passed parameters, and sends it back as a PNG
     string to the frontend for display.
@@ -717,7 +717,7 @@ def get_graph(event, wsc):
             return ''
 
         # Create a copy of the dataframe, just for safety.
-        df: pd.DataFrame = wsc.dfs[sheet_index].copy()
+        df: pd.DataFrame = steps_manager.dfs[sheet_index].copy()
 
         # Handle the graphs in alphabetical order
         if graph_type == BAR:
