@@ -12,7 +12,7 @@ from typing import Tuple, Union
 import pandas as pd
 import numpy as np
 
-from mitosheet.sheet_functions.types.utils import get_mito_type
+from mitosheet.sheet_functions.types.utils import BOOLEAN_SERIES, DATETIME_SERIES, NUMBER_SERIES, STRING_SERIES, get_mito_type
 
 def to_datetime_series(
         unknown_object,
@@ -28,13 +28,13 @@ def to_datetime_series(
         unknown_object = pd.Series([unknown_object])
         from_type = get_mito_type(unknown_object)
 
-    if from_type == 'boolean_series':
+    if from_type == BOOLEAN_SERIES:
         return None
-    elif from_type == 'datetime_series':
+    elif from_type == DATETIME_SERIES:
         return unknown_object
-    if from_type == 'number_series':
+    if from_type == NUMBER_SERIES:
         return None
-    elif from_type == 'string_series':
+    elif from_type == STRING_SERIES:
         # TODO: improve this to work element wise!
         return pd.to_datetime(unknown_object)
     else:
