@@ -1,26 +1,25 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# Copyright (c) Saga Inc.
-# Distributed under the terms of the GPL License.
+# Copyright (c) Mito.
+# Distributed under the terms of the Modified BSD License.
+
 from copy import deepcopy
-from time import perf_counter
-from typing import Any, Dict, List, Optional, Set, Tuple
-
-import pandas as pd
-from mitosheet.code_chunks.code_chunk import CodeChunk
-from mitosheet.code_chunks.step_performers.column_steps.refresh_dependant_columns_code_chunk import RefreshDependantColumnsCodeChunk
-from mitosheet.errors import (MitoError, make_circular_reference_error,
-                              make_execution_error, make_no_column_error,
-                              make_operator_type_error,
-                              make_unsupported_function_error)
-from mitosheet.parser import parse_formula
-from mitosheet.sheet_functions import FUNCTIONS
-from mitosheet.state import State
 from mitosheet.step_performers.step_performer import StepPerformer
-from mitosheet.evaluation_graph_utils import (create_column_evaluation_graph, creates_circularity, topological_sort_dependent_columns)
-from mitosheet.types import ColumnHeader, ColumnID
+from typing import Any, Dict, List, Optional
+from mitosheet.state import State
+from mitosheet.sheet_functions.types.utils import get_mito_type
+from mitosheet.topological_sort import creates_circularity, subgraph_from_starting_column_header, topological_sort_columns
+from mitosheet.sheet_functions import FUNCTIONS
+from mitosheet.parser import parse_formula
 
+from mitosheet.errors import (
+    make_circular_reference_error, 
+    make_no_column_error, 
+    make_operator_type_error, 
+    make_unsupported_function_error, 
+    make_wrong_column_metatype_error
+)
 
 class SetColumnFormulaStepPerformer(StepPerformer):
     """
@@ -31,19 +30,25 @@ class SetColumnFormulaStepPerformer(StepPerformer):
 
     @classmethod
     def step_version(cls) -> int:
-        return 2
+        return 1
 
     @classmethod
     def step_type(cls) -> str:
         return 'set_column_formula'
 
     @classmethod
-    def saturate(cls, prev_state: State, params: Dict[str, Any]) -> Dict[str, Any]:
+    def step_display_name(cls) -> str:
+        return 'Set Column Formula'
+    
+    @classmethod
+    def step_event_type(cls) -> str:
+        return 'set_column_formula_edit'
+
+    @classmethod
+    def saturate(cls, prev_state: State, params) -> Dict[str, str]:
         sheet_index = params['sheet_index']
-        column_id = params['column_id']
-        params['old_formula'] = prev_state.column_spreadsheet_code[sheet_index][column_id]
-        column_header = prev_state.column_ids.get_column_header_by_id(sheet_index, column_id)
-        column_headers = prev_state.dfs[sheet_index].keys()
+        column_header = params['column_header']
+        params['old_formula'] = prev_state.column_spreadsheet_code[sheet_index][column_header]
         
         # If the user submits an empty formula, we just set it equal to zero
         if params['new_formula'] == '':
@@ -52,120 +57,172 @@ class SetColumnFormulaStepPerformer(StepPerformer):
             try:
                 # Try and parse the formula, letting it throw errors if it
                 # is invalid
-                parse_formula(params['new_formula'], column_header, column_headers, throw_errors=True)
+                parse_formula(params['new_formula'], column_header, throw_errors=True)
             except:
-                params['new_formula'] = _get_fixed_invalid_formula(params['new_formula'], column_header, column_headers)
+                params['new_formula'] = _get_fixed_invalid_formula(params['new_formula'], column_header)
 
         # By default, we don't do anything with the saturate
         return params
 
     @classmethod
-    def execute( # type: ignore
+    def execute(
         cls,
         prev_state: State,
         sheet_index: int,
-        column_id: ColumnID,
+        column_header: str,
         old_formula: str,
         new_formula: str,
         **params
-    ) -> Tuple[State, Optional[Dict[str, Any]]]:
-        column_header = prev_state.column_ids.get_column_header_by_id(sheet_index, column_id)
+    ) -> State:
+        if column_header not in prev_state.column_metatype[sheet_index]:
+            raise make_no_column_error([column_header])
+
+        # First, we check the column_metatype, and make sure it's a formula
+        if prev_state.column_metatype[sheet_index][column_header] != 'formula':
+            raise make_wrong_column_metatype_error(column_header)
 
         # If nothings changed, there's no work to do
         if (old_formula == new_formula):
-            return prev_state, None
-
-        column_headers = prev_state.dfs[sheet_index].keys()
+            return
 
         # Then we try and parse the formula
-        _, new_functions, new_dependencies_column_headers = parse_formula(
+        new_python_code, new_functions, new_dependencies = parse_formula(
             new_formula, 
-            column_header,
-            column_headers
+            column_header
         )
-        new_dependencies = set(prev_state.column_ids.get_column_ids(sheet_index, new_dependencies_column_headers))
-
+        
         # We check that the formula doesn't reference any columns that don't exist
-        missing_columns = new_dependencies.difference(prev_state.column_spreadsheet_code[sheet_index].keys())
+        missing_columns = new_dependencies.difference(prev_state.column_metatype[sheet_index].keys())
         if any(missing_columns):
-            raise make_no_column_error(missing_columns, error_modal=False)
+            raise make_no_column_error(missing_columns)
 
         # The formula can only reference known formulas
         missing_functions = new_functions.difference(set(FUNCTIONS.keys()))
         if any(missing_functions):
-            raise make_unsupported_function_error(missing_functions, error_modal=False)
+            raise make_unsupported_function_error(missing_functions)
 
         # Then, we get the list of old column dependencies and new dependencies
         # so that we can update the graph
-        _, _, old_dependencies_column_headers = parse_formula(old_formula, column_header, column_headers)
-        old_dependencies = set(prev_state.column_ids.get_column_ids(sheet_index, old_dependencies_column_headers))
-
-        column_evaluation_graph = create_column_evaluation_graph(prev_state, sheet_index)
+        old_python_code, old_functions, old_dependencies = parse_formula(old_formula, column_header)
 
         # Before changing any variables, we make sure this edit didn't
         # introduct any circularity
         circularity = creates_circularity(
-            column_evaluation_graph, 
-            column_id,
+            prev_state.column_evaluation_graph[sheet_index], 
+            column_header,
             old_dependencies,
             new_dependencies
         )
         if circularity:
-            raise make_circular_reference_error(error_modal=False)
+            raise make_circular_reference_error()
 
         # We check out a new step
-        post_state = prev_state.copy(deep_sheet_indexes=[sheet_index])
+        post_state = deepcopy(prev_state)
 
         # Update the column formula, and then execute the new formula graph
-        try:
-            post_state.column_spreadsheet_code[sheet_index][column_id] = new_formula
-            pandas_start_time = perf_counter()
-            refresh_dependant_columns(post_state, post_state.dfs[sheet_index], sheet_index, column_id)
-            pandas_processing_time = perf_counter() - pandas_start_time
-        except MitoError as e:
-            # Catch the error and make sure that we don't set the error modal
-            e.error_modal = False
-            raise e
-        except:
-            raise make_execution_error(error_modal=False)
+        _update_column_formula_in_step(post_state, sheet_index, column_header, old_formula, new_formula)
+        _execute(post_state, post_state.dfs[sheet_index], sheet_index)
 
-        return post_state, {
-            'pandas_processing_time': pandas_processing_time
-        }
+        # Finially, update the type of the filters of this column, for all the filters
+        # TODO: fix bug where we have to update downstream types, but note that
+        # it would just be really nice if we didn't have to store the type here, and could
+        # just get it dynamically...
+        new_type = get_mito_type(post_state.dfs[sheet_index][column_header])
+        post_state.column_type[sheet_index][column_header] = new_type
+        post_state.column_filters[sheet_index][column_header]['filters'] = [
+            {'type': new_type, 'condition': filter_['condition'], 'value': filter_['value']} 
+            for filter_ in prev_state.column_filters[sheet_index][column_header]['filters']
+        ]
+
+        return post_state, None
 
     @classmethod
     def transpile(
         cls,
         prev_state: State,
         post_state: State,
-        params: Dict[str, Any],
         execution_data: Optional[Dict[str, Any]],
-    ) -> List[CodeChunk]:
+        sheet_index: int,
+        column_header: str,
+        old_formula: str,
+        new_formula: str
+    ) -> List[str]:
         """
         Transpiles an set_column_formula step to python code!
         """
-        return [
-            RefreshDependantColumnsCodeChunk(prev_state, post_state, params, execution_data)
-        ]
+        code = []
+
+        # We only look at the sheet that was changed, and sort the columns, taking only
+        # those downstream from the changed columns
+        subgraph = subgraph_from_starting_column_header(post_state.column_evaluation_graph[sheet_index], column_header)
+        topological_sort = topological_sort_columns(subgraph)
+
+        # We compile all of their formulas
+        for column in topological_sort:
+            column_formula_changes = post_state.column_python_code[sheet_index][column]
+            if column_formula_changes != '':
+                # We replace the data frame in the code with it's parameter name!
+                # NOTE: we check for df[ to increase the odds that we don't replace
+                # something other than the dataframe name itself (e.g. replacing a column
+                # name with the letters "df" inside of them
+                column_formula_changes = column_formula_changes.strip().replace('df[', f'{post_state.df_names[sheet_index]}[')
+                code.append(column_formula_changes)
+
+        return code
 
     @classmethod
-    def get_modified_dataframe_indexes( # type: ignore
-        cls, 
+    def describe(
+        cls,
         sheet_index: int,
-        column_id: ColumnID,
+        column_header: str,
         old_formula: str,
         new_formula: str,
+        df_names=None,
         **params
-    ) -> Set[int]:
-        return {sheet_index}
-
-
-
-def _get_fixed_invalid_formula(
-        new_formula: str, 
-        column_header: ColumnHeader, 
-        column_headers: List[ColumnHeader]
     ) -> str:
+        if df_names is not None:
+            df_name = df_names[sheet_index]
+            return f'Set {column_header} in {df_name} to {new_formula}'
+        return f'Set {column_header} to {new_formula}'
+
+
+def _update_column_formula_in_step(
+        post_state: State,
+        sheet_index: int,
+        column_header: str,
+        old_formula: str,
+        new_formula: str
+    ):
+    """
+    A  helper function for updating the formula of a column. It assumes
+    that the passed information is all _correct_ and will not:
+    1. Introduce a circular reference error.
+    2. Add an invalid formula.
+
+    It DOES NOT "reexecute" the dataframes, just updates the state variables.
+    """
+
+    new_python_code, _, new_dependencies = parse_formula(
+        new_formula, 
+        column_header
+    )
+
+    _, _, old_dependencies = parse_formula(
+        old_formula, 
+        column_header
+    )
+
+    post_state.column_spreadsheet_code[sheet_index][column_header] = new_formula
+    post_state.column_python_code[sheet_index][column_header] = new_python_code
+
+    # Update the column dependency graph
+    for old_dependency in old_dependencies:
+        post_state.column_evaluation_graph[sheet_index][old_dependency].remove(column_header)
+    for new_dependency in new_dependencies:
+        post_state.column_evaluation_graph[sheet_index][new_dependency].add(column_header)
+
+
+def _get_fixed_invalid_formula(new_formula, address):
     """
     A helper function that, given a formula, will try and fix
     any common errors with it. 
@@ -187,7 +244,7 @@ def _get_fixed_invalid_formula(
     for fixed_formula in POTENTIAL_VALID_FORMULAS:
         try:
             # Parse the formula, and return if it is valid
-            parse_formula(fixed_formula, column_header, column_headers, throw_errors=True)
+            parse_formula(fixed_formula, address, throw_errors=True)
             return fixed_formula
         except:
             pass
@@ -195,7 +252,7 @@ def _get_fixed_invalid_formula(
     return new_formula
 
 
-def get_details_from_operator_type_error(error: TypeError) -> Optional[Tuple[str, str, str]]:
+def get_details_from_operator_type_error(error):
     """
     We detect operator errors by checking the error string, which has the format:
 
@@ -210,9 +267,6 @@ def get_details_from_operator_type_error(error: TypeError) -> Optional[Tuple[str
     If you write pd.Series(['true']) ^ 10
     - Cannot perform 'xor' with a dtyped [object] array and scalar of type [bool]
 
-    On Python 3.6, there is one error when you try and append a number to 
-    a string, where it says `must be str, not int`.
-
     ^ NOTE: the above two errors makes pretty much no sense at all, but it appears
     to be casting the number to a boolean? So we choose to handle this as a number,
     as this is when you would be 
@@ -231,7 +285,7 @@ def get_details_from_operator_type_error(error: TypeError) -> Optional[Tuple[str
         return (operator, arg_one_type, arg_two_type)
     # Handle case where pandas dataframes are part of the operator, for a concatenate
     if error_message.startswith('can only concatenate str'):
-        # We cannot tell the order here (e.g. we don't know if a string was concated
+        # We cannot tell the ordered here (e.g. we don't know if a string was concated
         # to an integer, or the other way around) b/c the error doesn't say so, so we just
         # do our best 
         other_type = error_message.split('"')[1]
@@ -242,36 +296,24 @@ def get_details_from_operator_type_error(error: TypeError) -> Optional[Tuple[str
         # casting that goes on in the error message that makes it hard to tell. As such,
         # we report the most likely error.
         return ('^', 'number', 'str')
-    if error_message.startswith('must be str, not'):
-        # When adding a number to a string
-        return ('+', 'str', 'number')
+
 
     return None
 
 
-def refresh_dependant_columns(post_state: State, df: pd.DataFrame, sheet_index: int, column_id: ColumnID) -> None:
+def _execute(post_state, df, sheet_index):
     """
-    Helper function for refreshing the columns that are dependant on the column we are changing. 
+    Executes the given state variables for  
     """
-    topological_sort = topological_sort_dependent_columns(post_state, sheet_index, column_id)
-    column_headers = post_state.dfs[sheet_index].keys()
 
-    for column_id in topological_sort:
-        if post_state.column_spreadsheet_code[sheet_index][column_id] == '':
-            continue
-
-        column_header = post_state.column_ids.get_column_header_by_id(sheet_index, column_id)
-        python_code, _, _ = parse_formula(
-            post_state.column_spreadsheet_code[sheet_index][column_id], 
-            column_header,
-            column_headers
-        )
+    topological_sort = topological_sort_columns(post_state.column_evaluation_graph[sheet_index])
 
+    for column_header in topological_sort:
         # Exec the code, where the df is the original dataframe
         # See explination here: https://www.tutorialspoint.com/exec-in-python
         try:
             exec(
-                python_code,
+                post_state.column_python_code[sheet_index][column_header],
                 {'df': df}, 
                 FUNCTIONS
             )
@@ -285,9 +327,4 @@ def refresh_dependant_columns(post_state: State, df: pd.DataFrame, sheet_index:
                 raise make_operator_type_error(*operator_type_error_details)
             else:
                 # If it's not an operator error, we just propagate the error up
-                raise e
-        except NameError as e:
-            # If we have a column header that does not exist in the formula, we may
-            # throw a name error, in which case we alert the user
-            column_header = str(e).split('\'')[1]
-            raise make_no_column_error({column_header})
\ No newline at end of file
+                raise e
\ No newline at end of file
