@@ -3,47 +3,65 @@
 
 # Copyright (c) Mito.
 # Distributed under the terms of the Modified BSD License.
-import json
 import os
 import pandas as pd
 import pytest
 
 from mitosheet.mito_widget import MitoWidget, sheet
-from mitosheet.tests.test_utils import create_mito_wrapper, create_mito_wrapper_dfs
+from mitosheet.utils import get_invalid_headers
 from mitosheet.transpiler.transpile import transpile
-from mitosheet.tests.decorators import pandas_post_1_only
-from mitosheet.utils import MAX_COLUMNS
 
 
 def test_example_creation_blank():
     df = pd.DataFrame()
-    w = MitoWidget(df)
+    w = MitoWidget(df, tutorial_mode=False)
 
 VALID_DATAFRAMES = [
     (pd.DataFrame()),
     (pd.DataFrame(data={'A': [1, 2, 3]})),
     (pd.DataFrame(data={'A0123': [1, 2, 3]})),
-    (pd.DataFrame(data={0: [1, 2, 3]})),
-    (pd.DataFrame(data={0.1: [1, 2, 3]})),
-    (pd.DataFrame(data={'A A A A': [1, 2, 3], 0: [1, 2, 3]})),
-    (pd.DataFrame(data={'         a         ': [1, 2, 3], '123': [1, 2, 3]})),
-    (pd.DataFrame(data={'## this is a test': [1, 2, 3], ' !': [1, 2, 3]})),
-    (pd.DataFrame(data={'TOTAL': [1, 2, 3], '#123': [1, 2, 3]})),
-    (pd.DataFrame(data={'NUMber': [1, 2, 3], '.,,': [1, 2, 3]})),
-    (pd.DataFrame(data={'this is a possible ! column header that could be there': [1, 2, 3], '.,,': [1, 2, 3]})),
-    (pd.DataFrame(data={1000.123123: [1, 2, 3], 52.100: [1, 2, 3]})),
 ]
 @pytest.mark.parametrize("df", VALID_DATAFRAMES)
 def test_sheet_creates_valid_dataframe(df):
     mito = sheet(df)
     assert mito is not None
-    assert list(mito.steps_manager.curr_step.dfs[0].keys()) == list(df.keys())
 
+NON_VALID_HEADER_DATAFRAMES = [
+    (pd.DataFrame(data={0: [1, 2, 3]}), ['0_']),
+    (pd.DataFrame(data={0.1: [1, 2, 3]}), ['0_1_']),
+    (pd.DataFrame(data={'A': [1, 2, 3], 0: [1, 2, 3]}), ['A', '0_']),
+    (pd.DataFrame(data={'A': [1, 2, 3], '000': [1, 2, 3]}), ['A', '000_']),
+    (pd.DataFrame(data={'A': [1, 2, 3], 'abc-123': [1, 2, 3]}), ['A', 'abc_123']),
+    (pd.DataFrame(data={'A': [1, 2, 3], '-123': [1, 2, 3]}), ['A', '_123']),
+    (pd.DataFrame(data={'A': [1, 2, 3], '-123_': [1, 2, 3]}), ['A', '_123_']),
+    (pd.DataFrame(data={'A': [1, 2, 3], '-abc_': [1, 2, 3]}), ['A', '_abc_']),
+    (pd.DataFrame(data={'123': [1, 2, 3], '-abc_': [1, 2, 3]}), ['123_', '_abc_']),
+    (pd.DataFrame(data={'ABC!DEF': [1, 2, 3], '123': [1, 2, 3]}), ['ABC_DEF', '123_']),
+    (pd.DataFrame(data={'ABC?DEF': [1, 2, 3], '123': [1, 2, 3]}), ['ABC_DEF', '123_']),
+    (pd.DataFrame(data={' ': [1, 2, 3], '123': [1, 2, 3]}), ['_', '123_']),
+    (pd.DataFrame(data={'-': [1, 2, 3], ' !': [1, 2, 3]}), ['_', '__']),
+    (pd.DataFrame(data={'##': [1, 2, 3], ' !': [1, 2, 3]}), ['numnum', '__']),
+    (pd.DataFrame(data={'#!?5': [1, 2, 3], '#123': [1, 2, 3]}), ['num__5', 'num123']),
+    (pd.DataFrame(data={'()': [1, 2, 3], '.,,': [1, 2, 3]}), ['__', '___']),
+    (pd.DataFrame(data={'nate': [1, 2, 3], '.,,': [1, 2, 3]}), ['nate', '___']),
+    (pd.DataFrame(data={'//': [1, 2, 3], '-': [1, 2, 3]}), ['__', '_']),
+]
+
+@pytest.mark.parametrize("df, new_keys", NON_VALID_HEADER_DATAFRAMES)
+def test_sheet_errors_during_non_string_headers(df, new_keys):
+    assert len(get_invalid_headers(df)) != 0
+    mito = sheet(df)
+    assert list(mito.steps_manager.curr_step.dfs[0].keys()) == new_keys
 
 def test_create_with_multiple_dataframes():
     mito = sheet(pd.DataFrame(), pd.DataFrame(), pd.DataFrame(), pd.DataFrame(data={'A': [1, 2, 3]}))
     assert mito is not None
 
+def test_errors_on_non_existing_analysis():
+    with pytest.raises(ValueError):
+        mito = sheet(saved_analysis_name='bork a dork')
+
+
 def test_can_call_sheet_with_just_filename():
     df = pd.DataFrame(data={'A': [1, 2, 3], 'B': [2, 3, 4]})
     df.to_csv('../1.csv', index=False)
@@ -86,31 +104,11 @@ def test_can_call_sheet_with_df_and_filename():
 
     code_container = transpile(mito.steps_manager)
 
-    assert code_container == [
+    assert code_container['code'] == [
         '# Read in filepaths as dataframes',
-        'df_1 = pd.read_csv(r\'../1.csv\')',
+        'df_1_csv = pd.read_csv(r\'../1.csv\')',
     ]
 
-def test_can_use_utf_16_when_passing_string():
-    df = pd.DataFrame(data={'A': [1, 2, 3], 'B': [2, 3, 4]})
-
-    encoding = 'UTF-16'
-    df.to_csv('test_file_path.csv', index=False, encoding=encoding)
-
-    # Create with no dataframes
-    mito = sheet('test_file_path.csv')
-    # And then import just a test file
-
-    assert mito.steps_manager.dfs[0].equals(df)
-
-    code_container = transpile(mito.steps_manager)
-
-    assert code_container == [
-        '# Read in filepaths as dataframes',
-        f'test_file_path = pd.read_csv(r\'test_file_path.csv\', encoding=\'{encoding}\')',
-    ]
-    os.remove('test_file_path.csv')
-
 
 def test_call_makes_copies():
     df = pd.DataFrame(data={'A A': [1, 2, 3]})
@@ -126,16 +124,4 @@ def test_can_call_with_indexes():
     sheet(string_index)
 
     multi_index = df.set_index(['B', 'D'])
-    sheet(multi_index)
-
-def test_sheet_json_holds_all_columns():
-    df = pd.DataFrame({i: [1, 2, 3] for i in range(MAX_COLUMNS + 100)})
-    mito = create_mito_wrapper_dfs(df)
-    sheet_data = json.loads(mito.mito_widget.sheet_data_json)[0]
-    for i in range(MAX_COLUMNS, MAX_COLUMNS + 100):
-        assert sheet_data['columnIDsMap'][str(i)] is not None
-        assert sheet_data['columnSpreadsheetCodeMap'][str(i)] is not None
-        assert sheet_data['columnFiltersMap'][str(i)] is not None
-        assert sheet_data['columnIDsMap'][str(i)] is not None
-        assert sheet_data['columnDtypeMap'][str(i)] is not None
-        assert sheet_data['columnFormatTypeObjMap'][str(i)] is not None
+    sheet(multi_index)
\ No newline at end of file
