@@ -4,13 +4,12 @@
 # Copyright (c) Mito.
 # Distributed under the terms of the Modified BSD License.
 
-import pytest
-
+from mitosheet.state import DATAFRAME_SOURCE_DUPLICATED, DATAFRAME_SOURCE_IMPORTED, DATAFRAME_SOURCE_MERGED, DATAFRAME_SOURCE_PASSED, DATAFRAME_SOURCE_PIVOTED
 from mitosheet.user.schemas import USER_JSON_DEFAULT
-from mitosheet.steps import STEPS
-from mitosheet.steps.pivot import PIVOT_AGGREGATION_TYPES
-from mitosheet.steps.sort import ASCENDING, DESCENDING
-from mitosheet.steps.filter import BOOLEAN_FILTER_CONDITIONS, FC_EMPTY, FC_NOT_EMPTY, STRING_FILTER_CONDITIONS, NUMBER_FILTER_CONDITIONS, DATETIME_FILTER_CONDITIONS
+from mitosheet.step_performers import STEP_PERFORMERS
+from mitosheet.step_performers.pivot import PIVOT_AGGREGATION_TYPES
+from mitosheet.step_performers.sort import ASCENDING, DESCENDING
+from mitosheet.step_performers.filter import BOOLEAN_FILTER_CONDITIONS, FC_EMPTY, FC_NOT_EMPTY, STRING_FILTER_CONDITIONS, NUMBER_FILTER_CONDITIONS, DATETIME_FILTER_CONDITIONS
 from mitosheet.sheet_functions.types.utils import BOOLEAN_SERIES, DATETIME_SERIES, NUMBER_SERIES, STRING_SERIES, TIMEDELTA_SERIES
 from mitosheet.data_in_mito import DataTypeInMito
 
@@ -114,7 +113,11 @@ def test_pivot_aggregation_functions_match():
 
 def test_step_types_match():
     step_types = get_enum_from_ts_file('./src/types/StepTypes.ts', 'StepType')
-    assert set(step_types.values()) == set([step_object['step_type'] for step_object in STEPS] + ['initialize'])
+    assert set(step_types.values()) == set([step_performers.step_type() for step_performers in STEP_PERFORMERS] + ['initialize'])
+
+def test_df_sources_matche():
+    df_sources = get_enum_from_ts_file('./src/types/DFSources.ts', 'DFSource')
+    assert set(df_sources.values()) == set([DATAFRAME_SOURCE_PASSED, DATAFRAME_SOURCE_IMPORTED, DATAFRAME_SOURCE_PIVOTED, DATAFRAME_SOURCE_MERGED, DATAFRAME_SOURCE_DUPLICATED])
 
 
 def test_user_json_fields_match():
