@@ -11,7 +11,7 @@ and which of them were file paths, and updates the
 df names in the steps properly.
 """
 
-from mitosheet.steps.import_steps.simple_import import get_dataframe_names
+from mitosheet.step_performers.import_steps.simple_import import get_dataframe_names
 
 ARGS_UPDATE_EVENT = 'args_update'
 ARGS_UPDATE_PARAMS = ['args']
@@ -27,7 +27,7 @@ def is_str_df_name(arg):
     return arg.startswith('\'') or arg.startswith('\"')
 
 def execute_args_update(
-        wsc,
+        steps_manager,
         args
     ):
     """
@@ -69,7 +69,7 @@ def execute_args_update(
         # Finially, we don't add more names than there are dataframes (as this is clearly
         # nonsense), and thus this allows us to filter out Nones that are passed at the 
         # end of the arguments (not creating phantom tabs that cannot be clicked)
-        wsc.curr_step['df_names'] = final_names[:len(wsc.curr_step['dfs'])]
+        steps_manager.curr_step.post_state.df_names = final_names[:len(steps_manager.curr_step.dfs)]
 
 
 ARGS_UPDATE = {
